#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

# This is a shim to allow installation via pip for users on older versions
# The actual build configuration is in pyproject.toml

if __name__ == "__main__":
    setup()
